#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 01-Jan-2007
#################################################################################
#
# Manage Gallery - Script part of Ethereal Realms, designed to manage galleries
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing handler
use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filtering handler
use Ethereal::Home;						# File handler
use Ethereal::Image;						# Image handling
use Ethereal::Leech;						# Leeching ability
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handler
use Ethereal::Menu;						# Consistent menu
use Ethereal::Option;						# Option handlings
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler
use Ethereal::Words;						# Words and language module

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 8192;					# Maximum posts
$CGI::DISABLE_UPLOADS = 0;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $image;							# Image handling handle
my $leech;							# Leeching handle
my $login;							# Login handle
my $menu;							# Menu handle
my $page;							# Page handling library
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $url;							# Self referencing link
my $access;							# Galleries exist
my $expire;							# Expired access
my $task;							# Designated task
my $upload;							# Upload task

my @are;							# Are in group
my @not;							# Not in group
my @genres;							# Applicable genres for gallery
my @works;							# List of works to eliminate

my %args;
my %contrib;							# Contributor hash
my %links;


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);

	# Kill switch
	exit if $database->{'COMP_GALLERY'} eq 'false';

	# Set title
	$database->{'TITLE'} = $database->{'SYS'}{'TitGallery'};


	# Prefetch uploads
	$upload	= $cgi->upload('UPLOAD_WORK');
		

	# Multiple parameters
	@genres = $cgi->param('GGENRES');
	@are	= $cgi->param('LARE');
	@not	= $cgi->param('LNOT');
	@works  = $cgi->param('WORK');


	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();


	# Fetch hash
	$database->GetHashContrib(\%contrib);

	# Create instances
	$tmpl = new Ethereal::Template(\%contrib);
	$menu = new Ethereal::Menu($database, $param);


	# Login management
	# Login handle
	$login    = new Ethereal::Login($database, $cgi, $param, $menu, \%contrib);

	# Enable custom header and footer
	$login->{'HEADER'} = $database->DocumentGetContribHeader();
	$login->{'FOOTER'} = $database->DocumentGetContribFooter();

	# Disable header generation
	$login->{'AUTOMATE'} = 0;


	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Retreive access conditions
		($expire) = $database->DataGetPuppeteerExpire($param->{'USER'});
		($access) = $database->DataGetGalleryGranted($param->{'USER'});

		# Verified
		if (($expire eq 'no')
		 || ($access > 0))
		{

			# Quick check
			# Required for leech
			$param->{'TASK'} = 'leech' 
			  if ((defined($param->{'LEECH_WORK'}))
			   && ($param->{'LEECH_WORK'} =~ /^http:\/\/.{7,}/i));

			# Safety
			$param->{'TASK'} = (defined($param->{'TASK'}))
		       	  ? $param->{'TASK'}
			  : 'gallist';

			# Cleaner assignment
			$task = $param->{'TASK'};


			# Generate passable links
			# Check for necessary subset
			if (defined($param->{'GID'}))
			{
				# Existence
				# Find the proper ID and not a name
				($param->{'GID'}) = $database->DataGetGalleryID($param->{'GID'})
			          if ($param->{'GID'} !~ /^\d+$/);


				# Capture url
				$url   = $cgi->url(-absolute=>1);

				# Argument handling
				%args = (
					USER  => $param->{'USER'}, 
					CRYPT => $param->{'CRYPT'}
				 );

				# Reset link back down a level
				$links{'LGMAIN'} = $url . $param->EmbedInline(%args);


				# Addition of Gallery ID
				$args{'GID'} = $param->{'GID'}
			       	  if (defined($param->{'GID'}));
				   
				# Clear reset without layer
				$links{'LGRSET'} = $url . $param->EmbedInline(%args, TASK=>'galview');


				# Layer adjustments
				$args{'GLAYER'} = $param->{'GLAYER'}
				  if (defined($param->{'GLAYER'}));

				# Links
				$links{'LWLIST'} = $url . $param->EmbedInline(%args, TASK=>'worklist'); 
				$links{'LGNEWS'} = $url . $param->EmbedInline(%args, TASK=>'news');
				$links{'LGLINK'} = $url . $param->EmbedInline(%args, TASK=>'link'); 
				$links{'LGACCS'} = $url . $param->EmbedInline(%args, TASK=>'access'); 
				$links{'LGVIEW'} = $url . $param->EmbedInline(%args, TASK=>'galview');
				$links{'LGLAYR'} = $url . $param->EmbedInline(%args, TASK=>'layer');
			}


			# Check tasks
			# None of these are leeching
			unless ($task eq 'leech')
			{
				# Cheat a bit
				# Create easier to pass list
				my @args = ($database, $cgi, $param, $tmpl, \%contrib, \%links);


				# Changing of modes
				if (($task eq 'gallist')
				 && (defined($param->{$contrib{'TxtDetails'}})))
			 	{
					# Reset option list
					$task = 'galview';

					# Reset accordingly
					$param->{'TASK'} = $task;
				}
				

				# This mode has no requirement for alternate headers
				# Print file header
				$login->WriteHeader();

				# Document header
				print $menu->Rewrite($login->{'HEADER'}), "\n";


				# Additional check
				if ((defined($param->{'GID'}))
				 && ($task ne 'gallist'))
				{
					# Works
					# Detailed view
					if (($task eq 'workview')
					 || ((defined($param->{'WORK'}))
					  && (defined($param->{$contrib{'TxtDetails'}}))))
					{ 
						# Detailed view of all works
						WorkDetail(@args);
					}

					# Summary View
					elsif ($task eq 'worklist')
					{
						# Summary listing of works
						WorkSummary(@args, $upload, \@works);
					}

	
					# Additional controls
					# News entries and updates
					elsif ($task eq 'news')
					{
						# News entry management
						NewsEntry(@args);
					}

					# Layer entry
					elsif ($task eq 'layer')
					{
						# High level layer management
						LayerEntry(@args);
					}					

					# Link entries and updates
					elsif ($task eq 'link')
					{
						# Link additions and removals
						LinkEntry(@args);
					}	

					# Access control
					elsif ($task eq 'access')
					{
						# Access control lists
						Access(@args);
					}
	
					# Gallery specific
					# Detailed view
					else
					{
						# Base view outside of selection
						# Lists layers as well as galleries
						GalleryDetail(@args, \@genres, \@are, \@not);
					}
				}
	
				# Default behaviour
				else
				{
					# Gallery listing
					GallerySummary(@args, $expire);
				}

	
				# Display footer
				print $login->{'FOOTER'}, "\n";
			}

			# Leech mode
			else
			{
				# Create instances
				$page  = new Ethereal::Home($database,  $database->{'SYS'}{'SetGalleryRoot'});
				$image = new Ethereal::Image($database, $database->{'SYS'}{'SetGalleryRoot'});

				# Leeching and writing
				$leech = new Ethereal::Leech($database, $image, $page, $param, \%contrib);

				
				# This mode requires an alternate header
				# due to autoflushing
				
				# Flush output
				$| = 1;
				print "";

				# File header
				$login->WriteHeader(undef, -target=>'Main');


				# Information display
				# Display footer
				print $menu->Rewrite($contrib{'TmplLeechHead'});

				# Let the leeching begin
				$leech->Leech($param->{'LEECH_WORK'});


				# Display footer
				$tmpl->Show('TmplLeechFoot',
					LGVIEW => $links{'LGVIEW'},
					LWLIST => $links{'LWLIST'}
				 );
			}
		}

		# Account of access has expired
		# Put in full headers
		else
		{
			# File header
			$login->WriteHeader();

			# Document header
			print $menu->Rewrite($login->{'HEADER'}), "\n";
			
			
			# Non verified
			$tmpl->Show('TmplMsg', MMESSAGE=>$contrib{'MsgExpire'});

	
			# Document footer
			print $login->{'FOOTER'}, "\n";
		}
	}

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Access Entry
#
# Allows a host to grant additional power to a user.  Essentially opening up the
# gallery to allow co-hosts et cetera.

sub Access
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $links    = shift;					# Hash of links

	my $res;						# Database results
	my $statement;						# Database statement
	
	my $acl;						# Access control list
	my $editor;						# Editor access
	my $expunge;						# Expunge widget
	my $janitor;						# Janitorial access
	my $short;						# Short name

	my $login;						# Login name
	

	#####################
	# Program area	

	# Quick check
	return 0 unless (PermissionOwner($database, $param, $tmpl, $links));


	# Insertion
	# Insert new account
	if ((defined($param->{'NEW_ACCESS'}))
	 && (length($param->{'NEW_ACCESS'}) > 2))
	{
		# Verification
		($login) = $database->DataGetPuppeteerLogin($param->{'NEW_ACCESS'}); 

		# Safe
		if (defined($login))
		{
			# Data integrity
			($short) = $database->DataGetPuppetExistence($param->{'NEW_ACCESS'}, $login);

			# Username exists
			$editor  = (defined($param->{"NEW_ACCESS_EDITOR"}))  ? 'yes' : 'no';
			$janitor = (defined($param->{"NEW_ACCESS_JANITOR"})) ? 'yes' : 'no';

			# Insert
			$database->Quick("INSERT INTO GalleryAccess
				(GalleryID,
				 PuppeteerLogin,
				 PuppetName,
				 AccessEditor,
				 AccessJanitor)
				VALUES (?,?,?,?,?)",

			 $param->{'GID'},
			 $login,
			 $short,
			 $editor,
			 $janitor
			);
		}
	}

	# Gather information
	# Initialize query
	$database->Pull(\$statement, "SELECT 
		 PuppeteerLogin AS \"PuppeteerLogin\",
	         PuppetName     AS \"PuppetName\",
		 AccessEditor   AS \"AccessEditor\",
		 AccessJanitor  AS \"AccessJanitor\"
		FROM GalleryAccess
		WHERE GalleryID=?
		  AND AccessOwner <> ?", $param->{'GID'}, 'yes');
	 	 

	# Initialize
	$acl = '';

	# Loop and display
	# Generate if necessary
	while ($res = $statement->fetchrow_hashref())
	{
		# Generate short name
		$short = uc($res->{'PuppeteerLogin'});
		$short =~ s/\W//gs;
			
		# Normal operations
		unless (defined($param->{"ACCESS_${short}_EXPUNGE"}))
		{
			# Username exists
			$editor  = (defined($param->{"ACCESS_${short}_EDITOR"}))  ? 'yes' : 'no';
			$janitor = (defined($param->{"ACCESS_${short}_JANITOR"})) ? 'yes' : 'no';


			# Change if necessary
			if (defined($param->{"ACCESS_${short}"}))
			{
				# Update database
				$database->Quick("UPDATE GalleryAccess
					SET AccessEditor=?,
					    AccessJanitor=?
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $editor,
				 $janitor,
				 $param->{'GID'},
				 $res->{'PuppeteerLogin'}
				);

				# Generate widgets
				$editor = ($editor eq 'yes')
				 ? $cgi->checkbox(-name=>"ACCESS_${short}_EDITOR", -label=>'', -checked=>'checked')
				 : $cgi->checkbox(-name=>"ACCESS_${short}_EDITOR", -label=>'');
				$janitor = ($janitor eq 'yes')
				 ? $cgi->checkbox(-name=>"ACCESS_${short}_JANITOR", -label=>'', -checked=>'checked')
				 : $cgi->checkbox(-name=>"ACCESS_${short}_JANITOR", -label=>'');
			}

			# Use defaults
			else
			{
				# Generate widgets
				$editor = ($res->{'AccessEditor'} eq 'yes')
				 ? $cgi->checkbox(-name=>"ACCESS_${short}_EDITOR", -label=>'', -checked=>'checked')
				 : $cgi->checkbox(-name=>"ACCESS_${short}_EDITOR", -label=>'');
				$janitor = ($res->{'AccessJanitor'} eq 'yes')
				 ? $cgi->checkbox(-name=>"ACCESS_${short}_JANITOR", -label=>'', -checked=>'checked')
				 : $cgi->checkbox(-name=>"ACCESS_${short}_JANITOR", -label=>'');
			}

			# Universal expunge widget
			$expunge = $cgi->checkbox(-name=>"ACCESS_${short}_EXPUNGE", -label=>'');
			
			# Append
			$acl .= $tmpl->Pass('TagGalLstACL',
				MPUPPET  => $res->{'PuppetName'} . $cgi->hidden("ACCESS_${short}", 'True'),
				WEDITOR  => $editor,
				WJANITOR => $janitor,
				WREVOKE  => $expunge
			 ) . "\n";
		}

		# Display nothing
		else
		{
			# Remove quickly
			$database->Quick("DELETE FROM GalleryAccess
				WHERE GalleryID=?
				  AND PuppeteerLogin=?",

			 $param->{'GID'},
	       		 $res->{'PuppeteerLogin'}
			);		 
		}
	}

	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Display template
	$tmpl->Show('TmplGalACL',
		LWLIST   => $links->{'LWLIST'},
		LGNEWS   => $links->{'LGNEWS'},
		LGLINK   => $links->{'LGLINK'},
		LGACCS   => $links->{'LGACCS'},
		LGLAYR   => $links->{'LGLAYR'},
		LGVIEW   => $links->{'LGVIEW'},
		LSTACL   => $acl,
		WPUPPET  => $cgi->textfield('NEW_ACCESS', '', 15, 30),
		WEDITOR  => $cgi->checkbox(-name=>"NEW_ACCESS_EDITOR",  -label=>''),
		WJANITOR => $cgi->checkbox(-name=>"NEW_ACCESS_JANITOR", -label=>''),
		WUPDATE  => $cgi->submit($contrib->{'TxtUpdate'})
	 );

	# End form
	print $cgi->end_form(), "\n";

	# Finish query
	$statement->finish() if (defined($statement));
}

#####################
# Check Data
#
# This subroutine exists to verify information that is sent to it.   Basically
# this makes a more consistent interface and simplifies matters.

sub CheckData
{
	#####################
	# Data members
	my $choice   = shift;					# Choice of the matter
	my $value    = shift;					# Value to verify
	my $limit    = shift;					# Optional minimum limit


	#####################
	# Program area

	# Text field check
	if ($choice == 0)
	{
		# Allows A-Z and spaces
		return 1 if ($value =~ /^\w[\w\s\'\"\.\,\:\;\?\-]{$limit,}$/);
	}

	# Email check
	elsif ($choice == 1)
	{
		# Has the semblence of an Email address
		return 1 if ($value =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s);
	}


	# If all else fail
	return 0;
}

# Alias for E-Eail validity
sub CheckEMail
{ CheckData(1, shift); }

# Alias for Lenght
sub CheckString
{ CheckData(0, shift, shift); }


#####################
# Publish
#
# An exact copy of the code that is used in the gallery.pl file in order to
# publish works.   This one is self-contained as to prevent any chance of tainting
# information.

sub Publish
{
	#####################
	# Data members

	my $page     = shift;					# Page handler
	my $results  = shift;					# Results 
	my $export   = shift; 					# Changeable template
	my $contrib  = shift; 					# Gallery hash

	my $file;						# Filename
	my $data;						# Data file

	
	#####################
	# Program area

	# Set filename
	$file = 'exp' . $results->{'WorkID'} . '.html';

	# Add in spaces
	$data =  $results->{'WorkText'};
	$data =~ s/\n/<BR>/gs;
	

	# Search and replace
	$export =~ s/MAUTHOR/$results->{'GalleryPenname'}/gs;
	$export =~ s/MDATE/$results->{'WorkTimestamp'}/gs;
	$export =~ s/MTITLE/$results->{'WorkName'}/gs;
	$export =~ s/MBODY/$data/gs;
						
	# Write temporary file
	$page->FileWrite($file, $export);


	# Return value
	return $file;
}


#####################
# Gallery Detail
#
# Gallery detail, will handle the finer points of a gallery itself.  This
# includes setting names, genres et cetera.

sub GalleryDetail
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Gallery hash
	my $links    = shift;					# Hash of links
	my $genres   = shift;					# Array of genres
	my $are      = shift;					# List of people in groups
	my $not      = shift;					# List of those outside of groups

	my $filter;						# Filter handler
	my $option;						# Option handler
	my $page;						# File writing library

	my $statement;						# Database query statement
	my $res;						# Results

	my $layers;						# Layers list

	my $exist;						# Existence of value
	my $genre;						# Individual genre
	my $time;						# Current time
	my $name;						# Name of a gallery/layer

	my $count = 0;						# Generic counter
	my $num;						# Number of works

	my $editor;						# Editor privs
	my $janitor;						# Janitoral privs
	my $owner;						# Owner privs
	my $puppet;						# Puppet owner

	my $type;						# Gallery type
	my $sub;						# Gallery sub-type

	my $comment;						# Commenting wigets
	my $display;						# Displauy settings
	my $privacy;						# Privacy wigets
	my $sort;						# Sorting wigets

	my $highlight;						# Highlighted work
	my $thumb;						# Thumbnail
	my $workid;						# Work identifier

	my $level;						# Unique level
	my @levels = qw(Zero One Two Three Four Five);		# Levels to use

	my @lare;						# List of who are in
	my @lnot;						# List of who are not
	my @works;						# Works selectable for highlight

	my @comment;						# Commenting controls
	my @estab;						# Established
	my @genre;						# Genres
	my @display;						# Display settings
	my @privacy;						# Privacy mode
	my @puppet;						# Puppet names
	my @rating;						# Ratings
	my @sort;						# Sorting control
	

	#####################
	# Program area	

	# Initiate instances
	$option = new Ethereal::Option(); 


	# Information pickup
	# Pull permissions
	($owner,
	 $editor,
	 $janitor,
	 $puppet) = $database->DataGetGalleryAccess($param->{'GID'}, $param->{'USER'});

	# Gallery Type
	($type, $sub) = $database->DataGetGalleryType($param->{'GID'});


	# Information modification
	# Creation of news items
	if ((defined($param->{'SAVE_NEWS'}))
	 && (length($param->{'SAVE_NEWS'}) > 15)
	 && (($owner eq 'yes') || ($editor eq 'yes')))
	{
		# Create instance
		$filter = new Ethereal::Filter();

		# Filter text
		$param->{'SAVE_NEWS'} = $filter->SafeHTML($param->{'SAVE_NEWS'});
		$param->{'SAVE_NEWS'} = substr(DBI::neat($param->{'SAVE_NEWS'}, 452), 1, -1);

		# Simplify HTML
		$param->{'SAVE_NEWS'} =~ s/\n/<BR>/gs;

		
		# Create entry
		$database->Quick("INSERT INTO Comment
			(GalleryID,
			 WorkID,
			 PuppetName,
			 PuppeteerLogin,
			 CommentPost,
			 CommentRating)
			VALUES(?,1,getPuppet(?),?,?,0)",

		 $param->{'GID'},
		 $param->{'USER'},
		 $param->{'USER'},
		 $param->{'SAVE_NEWS'}
		);

		# Safety cleanup
		$param->Cleanup('EDIT_GALLERY');
	}
	
	# Modify existing information
	if ((defined($param->{'EDIT_GALLERY'}))
	 && ($param->{'EDIT_GALLERY'} eq 'True')
 	 && ($owner eq 'yes'))
	{
		# Initial type setup
		if ((defined($param->{'GTYPE'}))
		 && ($param->{'GTYPE'} =~ /^([A-Z]+?)\.(.*)/))
		{
			# Evaluate block
			eval
			{
				# Assign
				$type = lc($1);
				$sub  = $2;

				# Update database
				$database->Write("UPDATE Gallery
					SET GalleryType=?,
					    GallerySub=?,
					    GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $type,
				 $sub,
				 $param->{'GID'},
				 $param->{'USER'}
				);

				# Add in necessary genre
				$database->Write("INSERT INTO GalleryGenre
					(GalleryID,
					 GenreName,
					 GenreType,
					 GenreSub)
					VALUES(?,?,?,?)",

				 $param->{'GID'},
				 $contrib->{'SetGenre'},
				 $type,
				 $sub
				);

				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}

			# Cleanup
			$param->Cleanup('GTYPE');
		}

		# Layer updates
		# Only appears if this window is called
		# from within a layer.
		elsif (defined($param->{'GLAYER'}))
		{
			# Initiate handle
			$filter = new Ethereal::Filter();


			# Check for existence
			($name)  = $database->DataGetLayerName($param->{'GID'}, $param->{'GLAYER'});

			# Highlighting
			($workid,
			 $highlight,
			 $thumb) = $database->DataGetImageThumbnail($param->{'LHLIGHT'}, $param->{'GID'})
			   if ($param->{'LHLIGHT'} ne $contrib->{'TxtNone'});


			# Description
			$param->{'LDESCR'} = $filter->StripHTML($param->{'LDESCR'});
			$param->{'LDESCR'} = substr(DBI::neat($param->{'LDESCR'}, 252), 1, -1);
			$param->{'LDESCR'} = (length($param->{'LDESCR'}) > 10) ? $param->{'LDESCR'} : undef;

			# Determine value
			# Short name
			my $sname    = (defined($exist)) 
				? $name 
				: $param->{'LSNAME'};

			# Full name
			my $fname    = (length($param->{'LFNAME'}) < length($sname)) 
				? $sname 
				: $param->{'LFNAME'};



			# Handling of have and have nots
			# Make sure it works
			eval
			{
				# Handling of member works
				# Did the user request the action
				if ((defined($param->{$contrib->{'TxtMoveIn'}}))
				  || (defined($param->{$contrib->{'TxtMoveOut'}})))
			  	{
					# Calculate amount of entries
					my $care = @{$are};
					my $cnot = @{$not};

					# There is something to do
					if ((defined($care))
					 && ($care > 0)
					 && (defined($param->{$contrib->{'TxtMoveIn'}})))
				   	{
						# Placeholders
						# Create
						my $pare = '?,' x $care;

						# Finalize
						chop($pare);
	

						# Updates
						# List of items in layer
						$database->Write("UPDATE Work
							SET WorkLayer=NULL
							WHERE GalleryID=?
							  AND WorkName IN ($pare)",

						  $param->{'GID'},
						  @{$are}
						);
					}

					# There is something to do
					if ((defined($cnot))
					 && ($cnot > 0)
					 && (defined($param->{$contrib->{'TxtMoveOut'}})))
					{
						# Placeholders
						# Create
						my $pnot = '?,' x $cnot;

						# Finalize
						chop($pnot);

					
						# List of items in layer
						$database->Write("UPDATE Work
							SET WorkLayer=?
							WHERE GalleryID=?
							  AND WorkName IN ($pnot)",
	
						  $param->{'GLAYER'},
						  $param->{'GID'},
						  @{$not}
						);
					}
				}

				# Main updates
				$database->Write("UPDATE Layer
					SET LayerShort=?,
					    LayerFull=?,
					    LayerDescription=?,
					    LayerWork=?,
					    LayerHighlight=?,
					    LayerThumbnail=?
					WHERE GalleryID=?
					  AND LayerID=?",

				 $sname,
				 $fname,
				 $param->{'LDESCR'},
				 $workid,
				 $highlight,
				 $thumb,
				 $param->{'GID'},
				 $param->{'GLAYER'}
				);

				# Maintain order
				foreach $level (@levels)
				{
					# Maintain order by tracking changes
					# to the short name
		                	$database->Write("UPDATE Layer
						SET   Level$level=?
						WHERE GalleryID=?
						  AND Level$level=?",
						
						$sname,
						$param->{'GID'},
						$name
					);
				};
				  
				# Commit changes
				$database->Commit();

			};

			# An error has occured
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}

		# Standard update features for
		# root updates
		else
		{
			# Initiate handle
			$filter = new Ethereal::Filter();


			# Existence
			($exist) = $database->DataGetGalleryID($param->{'GNAME'});
			($name)  = $database->DataGetGalleryName($param->{'GID'});

			# Highlighting
			# More to select from
			($workid,
			 $highlight,
			 $thumb) = $database->DataGetImageThumbnail($param->{'GHLIGHT'}, $param->{'GID'})
			   if ($param->{'GHLIGHT'} ne $contrib->{'TxtNone'});


			# Main update
			# Description
			$param->{'GDESCR'} = $filter->StripHTML($param->{'GDESCR'});
			$param->{'GDESCR'} = substr(DBI::neat($param->{'GDESCR'}, 252), 1, -1);
			$param->{'GDESCR'} = (length($param->{'GDESCR'}) > 15) ? $param->{'GDESCR'} : undef;

			# Determin value
			$name    = (!defined($exist)) ? $param->{'GNAME'} : $name;

			# Options
			$comment = $option->GalleryComment($contrib->{'OptComment'}, $param->{'GCOMMENT'});
			$privacy = $option->GalleryPrivacy($contrib->{'OptPrivacy'}, $param->{'GPRIVACY'});
			$sort    = $option->GallerySort($contrib->{'OptSort'}, $param->{'GSORT'});


			# Update
			eval
			{
				# Entry
				$database->Write("UPDATE Gallery
					SET GalleryName=?,
					    GalleryRating=?,
					    GalleryDescription=?,
					    GalleryWork=?,
					    GalleryHighlight=?,
					    GalleryThumbnail=?,
					    GalleryPenName=?,
					    GalleryDisplay=?,
			 		    GalleryComments=?,
			 		    GalleryPrivacy=?,
					    GallerySort=?,
					    GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",
				
			 	 $name,
				 $param->{'GRATING'},
				 $param->{'GDESCR'},
				 $workid,
				 $highlight,
				 $thumb,
				 $param->{'GPUPPET'},
				 $param->{'GDISPLAY'},
				 $comment,
				 $privacy,
				 $sort,
				 $param->{'GID'},
				 $param->{'USER'}
				);

				# Update access
				$database->Write("UPDATE GalleryAccess
					SET PuppetName=?
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $param->{'GPUPPET'},
				 $param->{'GID'},
				 $param->{'USER'}
				);

				# Update master work
				$database->Write("UPDATE Work
					SET WorkName=?
					WHERE WorkID < 2
					  AND GalleryID=?",

				 $name,
				 $param->{'GID'},
				);


				# Genre handling
				$database->Write("DELETE
					FROM GalleryGenre
					WHERE GalleryID=?",

				 $param->{'GID'},
			 	);


				# Loop and re-add
				foreach $genre (@{$genres})
				{
					# Exit loop if necessary
					last if ($count > 2);

					# Insert individual choices
					$database->Write("INSERT INTO GalleryGenre
						(GalleryID,
						 GenreName,
						 GenreType,
						 GenreSub)
						VALUES(?,?,?,?)",

				 	 $param->{'GID'},
					 $genre,
					 $type,
					 $sub
					);

					# Increment
					$count++;
				}
			
				# Commit transaction
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}
	}

	# Contributor specific information
	if ((defined($param->{'EDIT_GALLERY'}))
	 && ($param->{'EDIT_GALLERY'} eq 'True')
 	 && ($owner ne 'yes'))
	{
		# Evaluate statement
		eval
		{
			# Update access
			$database->Write("UPDATE GalleryAccess
				SET PuppetName=?
				WHERE GalleryID=?
				  AND PuppeteerLogin=?",

			 $param->{'GPUPPET'},
			 $param->{'GID'},
			 $param->{'USER'}
			);
		};

		# If block failed
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback();
		}
	}


	# Verify initial access
	if ($type eq 'initial')
	{
		# Pull appropriate records
		$database->Pull(\$statement, "SELECT
			  GenreName        AS \"GenreName\",
			  GenreType        AS \"GenreType\",
			  GenreDescription AS \"GenreDescription\"
			FROM Genre
			WHERE GenreType IN ('image','text')
			  AND GenreSub='<root>'
			ORDER BY GenreType, GenreSub",
  		);

		# Genre
		my $img_genre  = '';
		my $txt_genre  = '';
		my $init_select = '';
		

		# Confirm there is something to work with
		# Data check
		while ($res = $statement->fetchrow_hashref())
		{
			# Create wiget
			# Create appropriate wiget
			$init_select = '<INPUT TYPE="radio" NAME="GTYPE" VALUE="'
				. uc($res->{'GenreType'}) . '.' 
				. $res->{'GenreName'}     . '" />';
		

			# Gallery types are appart
			# Image galleries
			if ($res->{'GenreType'} eq 'image')
			{
				# Image based ones
				$img_genre .= $tmpl->Pass('TmplGalInitRow',
					MTITLE  => $res->{'GenreName'},
					MDESCR  => $res->{'GenreDescription'},
					WSELECT => $init_select
				 );
			}

			# Text based galleries
			else
			{
				# Image based ones
				$txt_genre .= $tmpl->Pass('TmplGalInitRow',
					MTITLE  => $res->{'GenreName'},
					MDESCR  => $res->{'GenreDescription'},
					WSELECT => $init_select
				 );
			}

		}

		# Finish query
		$statement ->finish();


		# Information display
		# Start Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_GALLERY => 'True');

		# Display initialization script
		$tmpl->Show('TmplGalInit',
			LSTIMAGE => $img_genre,
			LSTTEXT  => $txt_genre,
			WSELECT  => $cgi->submit($contrib->{'TxtSelect'})
		 );

		# Finish form
		print $cgi->end_form, "\n";

		# Return and avoid the rest
		return 1;
	}


	# Generate final template
	$layers = LayerGenerate($database, $param, $tmpl, $contrib, $links);
	

	# Check for the presence of layers
	# Standard root gallery settings
	unless (defined($param->{'GLAYER'}))
	{
		# Information gathering
		# Prepare root gallery information
		$database->Pull(\$statement, "SELECT
			 GalleryName        AS \"GalleryName\",
			 GalleryRating      AS \"GalleryRating\",
			 GalleryDescription AS \"GalleryDescription\",
			 GalleryHighlight   AS \"GalleryHighlight\",
			 GalleryComments    AS \"GalleryComments\",
			 GallerySort        AS \"GallerySort\",
			 GalleryType        AS \"GalleryType\",
			 GallerySub         AS \"GallerySub\",
			 GalleryPrivacy     AS \"GalleryPrivacy\",
			 GalleryDisplay     AS \"GalleryDisplay\",
			 getDate(GalleryCreated) AS \"GalleryCreated\",
			 getDate(GalleryUpdated) AS \"GalleryUpdated\"
			FROM Gallery
			WHERE GalleryID=?", 
		
		 $param->{'GID'}
 		);


		# Fetch results
		return 0 unless (($res) = $statement->fetchrow_hashref());


		# Information display
		# Start Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_GALLERY => 'True');

		

		# Default behaviour
		# Owner level
		if ($owner eq 'yes')
		{
			# Genres
			# Established
			$database->GetListEstablished(\@estab, $param->{'GID'});

			# Total
			$database->GetListGenre(\@genre, $type, $sub);	


			# Gather information
			$option->Split(\@comment, $contrib->{'OptComment'});
			$option->Split(\@display, $contrib->{'OptDisplay'});
			$option->Split(\@privacy, $contrib->{'OptPrivacy'});
			$option->Split(\@rating,  $contrib->{'OptRating'});
			$option->Split(\@sort,    $contrib->{'OptSort'});

			# Puppet
			$database->GetListPuppetYours(\@puppet, $param->{'USER'});


			# Works
			# Generate initial list
			$database->GetList(\@works, "SELECT WorkName
				 FROM Work
				 WHERE GalleryID=?
				   AND WorkLayer IS NULL
				   AND WorkID > 2",

			 $param->{'GID'}
			);

			# What did we get
			$num = @works;
			
			# Nothing to highlight
			if ($num < 1)
			{
				# Whatever a parent has this can now use
				$database->GetList(\@works, "SELECT LayerHighlight
					 FROM Layer
					WHERE LayerLevel=0
					  AND GalleryID=?
					  AND LayerHighlight IS NOT NULL
					ORDER BY LayerHighlight",

				  $param->{'GID'}
				);
			}

			# Append
			unshift(@works, $contrib->{'TxtNone'});


			# Initial discovery
			my $comment = $option->GalleryCommentRev(
				$contrib->{'OptComment'},
				$res->{'GalleryComments'}
			 );
			my $privacy = $option->GalleryPrivacyRev(
				$contrib->{'OptPrivacy'},
				$res->{'GalleryPrivacy'}
			 );
			my $sort    = $option->GallerySortRev(
				$contrib->{'OptSort'},
				$res->{'GallerySort'}
			 );
	
			# Generate widgets
			# Pen name and rating selection and highlights
			my $puppet  = (defined($database->{'DEFAULT'}))
			  ? $cgi->hidden('GPUPPET', $database->{'DEFAULT'})
			  : $cgi->popup_menu('GPUPPET',   \@puppet,  $puppet);
			my $rating  = $cgi->popup_menu('GRATING',   \@rating,  $res->{'GalleryRating'});
			my $glight  = $cgi->popup_menu('GHLIGHT',   \@works,  $res->{'GalleryHighlight'});

			# Checkbox group for genres
			my $genre   = $cgi->checkbox_group(
			  -name      => 'GGENRES',
		    	  -values    => \@genre,
			  -defaults  => \@estab,
		  	  -linebreak => 'true'
		  	 );

		  	# Comments, display settings
			# Privacy and sorting
			$comment = $cgi->radio_group('GCOMMENT', \@comment, $comment, 'true');
			$display = $cgi->radio_group('GDISPLAY', \@display, $res->{'GalleryDisplay'}, 'true');
			$privacy = $cgi->radio_group('GPRIVACY', \@privacy, $privacy, 'true'); 
			$sort    = $cgi->radio_group('GSORT',    \@sort,    $sort, 'true');

			# Additional controls
			# Gallery name
			my $gname = $cgi->textfield('GNAME', $res->{'GalleryName'}, 25, 30);
			
			# Gallery descriptions
			my $gdescr = $cgi->textarea(
			  -name    => 'GDESCR',
			  -rows    => 3,
			  -columns => 75,
			  -wrap    => 'SOFT',
			  -default => $res->{'GalleryDescription'}
		  	);
			
		  	# Submit and reset
			my $wsubmit = $cgi->submit($contrib->{'TxtUpdate'});
		       	my $wreset  = $cgi->reset($contrib->{'TxtReset'});


			# Information display
			# Display template
			$tmpl->Show('TmplGalDetail',
				LWLIST    => $links->{'LWLIST'},
				LGMAIN    => $links->{'LGMAIN'},
				LGNEWS    => $links->{'LGNEWS'},
				LGLINK    => $links->{'LGLINK'},
				LGACCS    => $links->{'LGACCS'},
				LGLAYR    => $links->{'LGLAYR'},
				LGVIEW    => $links->{'LGVIEW'},
				MGNAME    => $res->{'GalleryName'},
				MTYPE     => $res->{'GallerySub'},
				MCREATED  => $res->{'GalleryCreated'},
				MMODIFIED => $res->{'GalleryUpdated'},
				MLAYERS   => $layers,
				WGCOMMENT => $comment,
				WGNAME    => $gname,
				WGPEN     => $puppet,
				WGRATE    => $rating,
				WGGENRE   => $genre,
				WGHIDE    => $privacy,
				WGDISPLAY => $display,
				WGSORT    => $sort,
				WGDESCR   => $gdescr,
				WGLIGHT   => $glight,
				WUPDATE   => $wsubmit,
				WRESET    => $wreset
			);
		}

		# Display a generic
		# Template
		else
		{
		  	# Submit and reset
			my $wsubmit = $cgi->submit($contrib->{'TxtUpdate'});
		       	my $wreset  = $cgi->reset($contrib->{'TxtReset'});

			# Information display
			# Display more generic template
			$tmpl->Show('TmplGalDetailLight',
				LWLIST    => $links->{'LWLIST'},
				LGMAIN    => $links->{'LGMAIN'},
				LGNEWS    => $links->{'LGNEWS'},
				LGLINK    => $links->{'LGLINK'},
				LGVIEW    => $links->{'LGVIEW'},
				MGNAME    => $res->{'GalleryName'},
				MTYPE     => $res->{'GallerySub'},
				MCREATED  => $res->{'GalleryCreated'},
				MMODIFIED => $res->{'Updated'},
				MLAYERS   => $layers,
				WGPEN     => $puppet,
				WUPDATE   => $wsubmit,
				WRESET    => $wreset
			 );
		}

		# End of form
		print $cgi->end_form(), "\n";
	}

	# Layers and lots of layers
	else
	{
		# Information gathering
		# Prepare root gallery information
		$database->Pull(\$statement, "SELECT
			 LayerShort       AS \"LayerShort\",
			 LayerFull        AS \"LayerFull\",
			 LayerDescription AS \"LayerDescription\",
			 LayerHighlight   AS \"LayerHighlight\",
			 LayerLevel       AS \"LayerLevel\"
			FROM Layer
			WHERE GalleryID=?
			  AND LayerID=?", 
		
		 $param->{'GID'},
		 $param->{'GLAYER'}
 		);

		# Quick check
		return 0 unless ($res = $statement->fetchrow_hashref());


		# Pull lists
		# Those who are
		$database->GetList(\@lare, "SELECT WorkName
			FROM Work
			WHERE GalleryID=?
			  AND WorkLayer=?
			  AND WorkID > 2
			ORDER BY WorkName",

		 $param->{'GID'},
		 $param->{'GLAYER'}
		);
		
		# Those who stand alone
		$database->GetList(\@lnot, "SELECT WorkName
			 FROM Work
			WHERE GalleryID=?
			  AND WorkLayer IS NULL
			  AND WorkID > 2
			ORDER BY WorkName",

		 $param->{'GID'}
		);


		# Highlighted works
		# Generate initial list
		$database->GetList(\@works, "SELECT WorkName
			  FROM Work
			 WHERE GalleryID=?
			   AND WorkLayer=?
			   AND WorkID > 2
			 ORDER BY WorkName",

		 $param->{'GID'},
		 $param->{'GLAYER'}
		);

		# What did we get
		$num = @works;
		

		# Move up and see
		if ($num < 1)
		{
			# Whatever a parent has this can now use
			$database->GetList(\@works, "SELECT LayerHighlight
				 FROM Layer
				WHERE LayerLevel > ?
				  AND Level$levels[$res->{LayerLevel}]=?
				  AND LayerHighlight IS NOT NULL
				ORDER BY LayerHighlight",

			  $res->{'LayerLevel'},
			  $res->{'LayerShort'}
			);
		}

		# Append
		unshift(@works, $contrib->{'TxtNone'});


		# Generate Controls
		# Scrolling Lists
		my $ware  = $cgi->scrolling_list('LARE', \@lare, $lare[0], 10, 'true');
		my $wnot  = $cgi->scrolling_list('LNOT', \@lnot, $lnot[0], 10, 'true');

		# Popup menu
		my $light = $cgi->popup_menu('LHLIGHT', \@works, $res->{'LayerHighlight'});

		# Control widget
		my $control = $cgi->submit($contrib->{'TxtMoveIn'}) . "<BR><BR>" . $cgi->submit($contrib->{'TxtMoveOut'});
		

		# Additional controls
		# Gallery name
		my $lsname = $cgi->textfield('LSNAME', $res->{'LayerShort'}, 20, 16);
		my $lfname = $cgi->textfield('LFNAME', $res->{'LayerFull'}, 40, 250);
			
		# Gallery descriptions
		my $ldescr = $cgi->textarea(
		  -name    => 'LDESCR',
		  -rows    => 3,
		  -columns => 75,
		  -wrap    => 'SOFT',
		  -default => $res->{'LayerDescription'});
			
	  	# Submit and reset
		my $wlsubmit = $cgi->submit($contrib->{'TxtUpdate'});
	       	my $wlreset  = $cgi->reset($contrib->{'TxtReset'});


		# Information display
		# Begin form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_GALLERY => 'True');

		# Display more generic template
		$tmpl->Show('TmplGalDetailLayer',
			LWLIST    => $links->{'LWLIST'},
			LGMAIN    => $links->{'LGMAIN'},
			LGNEWS    => $links->{'LGNEWS'},
			LGLINK    => $links->{'LGLINK'},
			LGACCS    => $links->{'LGACCS'},
			LGLAYR    => $links->{'LGLAYR'},
			LGVIEW    => $links->{'LGVIEW'},
			MNAME     => $res->{'LayerShort'},
			MLAYERS   => $layers,
			WSHORT    => $lsname,
			WFULL     => $lfname,
			WDESCR    => $ldescr,
			WLIGHT    => $light,
			WARE      => $ware,
			WNOT      => $wnot,
			WCONTROL  => $control,
			WUPDATE   => $wlsubmit,
			WRESET    => $wlreset
		 );

		# End of form
		print $cgi->end_form(), "\n";
	}

	# Finish query
	$statement->finish();
}


#####################
# Gallery Summary
#
# Gallery summary is used to display a list of all galleries for the
# user and allow them to modify them as need be. 

sub GallerySummary
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Gallery hash
	my $links    = shift;					# Hash of links
	my $expire   = shift;					# Determins access

	my $audit;						# Auditing handle
	my $filter;						# Filter handle
	my $page;						# File handler

	my $buttons;						# Management buttons
	my $create;						# Creation ability
	my $listing;						# List of galleries

	my $exist;						# Existence
	my $id;							# Last insert ID
	my $message;						# Notice message
	my $puppet;						# Default puppet name
	my $work;						# Work list

	my @works;						# List of works
	my @galleries;						# Array of galleries


	#####################
	# Program area	

	# Initialize value
	$id = $param->{'GID'};


	# Information handling
	# Owners only
	if ($expire eq 'no')
	{
		# Create instance
		$audit = new Ethereal::Audit($database, $cgi, $param);

		# Additions with names greater then 3 
		if ((defined($param->{$contrib->{'TxtCreate'}}))
		 && (CheckString($param->{'ADD_GALLERY'}, 2)))
		{
			# Check for existence
			($exist)  = $database->DataGetGalleryID($param->{'ADD_GALLERY'});

			# Name not already taken
			unless (defined($exist))
			{
				# Evaluate block
				eval
				{
					# Retreive base information
					($puppet) = $database->DataGetDefault($param->{'USER'});

					# Create instance
					$page = new Ethereal::Home($database);

	
					# Creation
					$database->Write("INSERT INTO Gallery
						(PuppeteerLogin,
						 GalleryName,
						 GalleryRating,
						 GalleryDisplay,
						 GalleryPenName)
						VALUES(?,?,?,?,?)",
	
					 $param->{'USER'},
					 $param->{'ADD_GALLERY'},
					 $contrib->{'SetRating'},
					 $contrib->{'SetDisplay'},
					 $puppet
					);
		
	
					# Determine ID
					($id) = $database->DataGet("SELECT CURRVAL('seqGallery')");
	
	
					# Additional linked tables
					# Create ownership entry
							$database->Write("INSERT INTO GalleryAccess
						(GalleryID,
						 PuppetName,
						 PuppeteerLogin,
						 AccessOwner)
						VALUES(?,?,?,'yes')",
	
					 $id,
					 $puppet,
					 $param->{'USER'}
					);
					
					# Create system level works
					# Gallery Comments
					$database->Write("INSERT INTO Work
						(GalleryID,
						 WorkID,
						 WorkName,
					 	 PuppeteerLogin)
						VALUES(?,0,?,?)",
	
					 $id,
					 $param->{'ADD_GALLERY'},
					 $param->{'USER'}
					);
	
					# Artists news
					$database->Write("INSERT INTO Work
						(GalleryID,
						 WorkID,
						 WorkName,
					 	 PuppeteerLogin)
						VALUES(?,1,?,?)",
	
					 $id,
					 $param->{'ADD_GALLERY'},
					 $param->{'USER'}
					);
	

					# Adapt basedir
					$page->{'BASEDIR'} = $database->{'SYS'}{'SetGalleryRoot'} . $page->{'SEP'} . $id;
	
					# Remove if exists
					# Directory
					if (-d $page->{'BASEDIR'})
					{
						# Remove directory
						$page->DirClear($page->{'BASEDIR'});
					}
	
					# File
					if (-f $page->{'BASEDIR'})
					{
						# Unlink
						unlink($page->{'BASEDIR'});
					}
	

					# Attempt to create
					unless ($page->DirCreate($page->{'BASEDIR'}))
					{
						# Rollback
						$database->Rollback();
	
						# Signal notice
						$message = $contrib->{'MsgError'} . $!;
					}
	
					else
					{
						# Auditing
						$audit->GalleryAdd($id);
	
						# Message of success
						$message = $contrib->{'MsgCreated'};
					}
	
					# Commit
					$database->Commit();
				};
	
				# If block failed
				if ($@)
				{
					# Issue warning
					warn("Transaction aborted: $@");
	
					# Undo incomplete changes
					$database->Rollback();
				}
			}
	
			# Notify
			else
			{
				# Existence message
				$message = $contrib->{'MsgExists'};
			}
		}
	
		# Removal
		if ((defined($param->{$contrib->{'TxtDelete'}}))
		 && (defined($param->{'CONFIRM'})))
		{
			# Confirm
			($exist) = $database->DataGetGalleryName($id);
	
	
			# Systematically remove
			if ((defined($id))
			 && (defined($exist)))
			{
				# Find list of all works in gallery
				$database->GetList(\@works, "SELECT WorkID
					FROM Work
					WHERE GalleryID=?", $id);


				# Loop and remove
				while ($work = shift(@works))
				{
					# Remove each and every offending entry
					$database->Quick("DELETE FROM Work
						WHERE WorkID=?", $work);
				}
			
				# Remove database entry
				$database->Quick("DELETE FROM Gallery
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",
				 $id,
				 $param->{'USER'}
				);
	

				# Create instance
				$page = new Ethereal::Home($database);

					
				# Remove directory
				$page->DirClear($database->{'SYS'}{'SetGalleryRoot'} . $page->{'SEP'} . $id);
	
	
				# Signal notice
				$message = $contrib->{'MsgRemoved'};
	
	
				# Record change
				$audit->GalleryRem($id);

				# Commit any stale changes
				$database->Commit();
			}
		}
	}

	# Always cleanup
	$param->Cleanup('CONFIRM');



	# Retrieve list
	$database->GetList(\@galleries, "SELECT g.GalleryName AS \"Gallery\" 
		 FROM Gallery g, GalleryAccess a 
		 WHERE g.GalleryID=a.GalleryID
		   AND a.PuppeteerLogin=?
	  	 GROUP BY g.GalleryID, g.GalleryName
		 ORDER BY g.GalleryName", 
		 
	 $param->{'USER'}
 	);


	# Information display
	# Widget generation
	# Listing of galleries
	$listing = $cgi->scrolling_list('GID', \@galleries, $galleries[0], 5);

	# Ability to remove
	if ($expire eq 'no')
	{
		# Owners
		$buttons = $cgi->submit($contrib->{'TxtDetails'}) . '<BR><BR>'
			.  $cgi->submit($contrib->{'TxtDelete'})
			.  $cgi->checkbox(-name=>'CONFIRM', -label=>'');

		# Ability to create
		$create  = $tmpl->Pass('TmplGalSelectCreate',
			WNEWGAL  => $cgi->textfield('ADD_GALLERY', '', 25, 30),
			WCREATE  => $cgi->submit($contrib->{'TxtCreate'})
		 );
	}
	else
	{
		# Regular users
		$buttons = $cgi->submit($contrib->{'TxtDetails'});

		# Create needs are obviously not needed
		$create  = $contrib->{'TagGalExpire'}; 
	}


	# Information display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplGalSelect',
		MNOTICE  => $message,
		MCREATE  => $create,
		WSELECT  => $listing,
		WCONTROL => $buttons
	 );

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Quick Layer Entry
#
# Layer entry allows for the creation and removal of layers within a specific
# gallery.

sub LayerEntry
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $links    = shift;					# Hash of links

	my $res;						# Results
	my $statement;						# SQL statement

	my $count;						# Standard counter
	my $id;							# Current ID
	my $parent;						# Parent to layer
	my $exists;						# Existence test
	my $level;						# Level to deal with

	my @levels;						# Levels for a parent
	my @growth;						# Layers with growth
	my @complete;						# Complete list of layers

	#####################
	# Program area	

	# Quick check
	return 0 unless (PermissionEditor($database, $param, $tmpl, $links));


	# Data management
	# Addition
	if (defined($param->{$contrib->{'TxtCreate'}}))
	{
		# Basic safety
		if ((defined($param->{'LAYERADD'}))
		 && (defined($param->{'LAYERROOT'})))
	 	{
			# Make it easier for them to figure out
			$param->{'LAYERADD'} =~ s/\W+/_/gs;

			# Good size
			if (length($param->{'LAYERADD'}) > 2)
			{
				# Check for existence
				($id) = $database->DataGetLayer($param->{'GID'}, $param->{'LAYERADD'});

			
				# Cannot have duplicates
				unless (defined($id))
				{
					# Parent knowledge
					$parent =  $param->{'LAYERROOT'};
					$parent  =~ s/^\-+?\s//;

					# Determine information
					($parent,
					 $level,
					 @levels) = $database->DataGetLayer($param->{'GID'}, $parent)
				 		if ($parent ne $contrib->{'TxtNone'});


				 	# Make sure it works
					eval
					{

						# Level management
						# Manage levels
						unless (defined($level))
						{
							# Initialize levels
							$level = -1;

							# Create array
							push(@levels,0,0,0,0,0,0);
						}

						# Increment to child level
						$level++;
						
						# Generate accordingly to sort
						$levels[$level] = $param->{'LAYERADD'};

					
						# Create with defaults
						# Create entry
						$database->Write("INSERT INTO Layer
							(GalleryID,
							 LayerShort,
							 LayerFull,
							 LayerLevel,
							 LevelZero,
							 LevelOne,
							 LevelTwo,
							 LevelThree,
							 LevelFour,
							 LevelFive)
							VALUES (?,?,?,?,?,?,?,?,?,?)",

						  $param->{'GID'},
						  $param->{'LAYERADD'},
						  $param->{'LAYERADD'},
						  $level,
						  @levels
						);

						# Commit changes
						$database->Commit();
					};

					# This failed
					if ($@)
					{
						# Silently rollback
						$database->Rollback();
					}
				}
			}
		}
	}	

	# More data management
	# Removals
	if (defined($param->{$contrib->{'TxtDelete'}}))
	{
		# Layer update
		$id =  $param->{'LAYERREM'};
		$id =~ s/^\-+?\s//; 

		# Confirm it exists
		($exists) = $database->DataGetLayer($param->{'GID'}, $id)
		 	if ($id ne $contrib->{'TxtNone'});
		
		# No need to proceed
		# If does not exist
		if (defined($exists))
		{
			# Make sure this works
			eval
			{
				# Check for all children
				$database->Pull(\$statement, "SELECT LayerID AS \"LayerID\"
					FROM Layer
					WHERE (LevelZero=?
					   OR LevelOne=?
					   OR LevelTwo=?
					   OR LevelThree=?
					   OR LevelFour=?
					   OR LevelFive=?)
					  AND GalleryID=?",
	
				  $id, $id, $id, $id, $id, $id, $param->{'GID'}
				 );

				# Loop and remove all who matter
				while (($id) = $statement->fetchrow())
				{
					# Remove layer
					# Order is irrelevant
					$database->Write("DELETE FROM Layer
						 WHERE GalleryID=?
						   AND LayerID=?",

					 $param->{'GID'},
					 $id
					);
				}

				# Complete search
				$statement->finish();

				# Commit changes
				$database->Commit();
			};

			# Something went wrong
			if ($@)
			{
				# Silently rollback
				$database->Rollback();
			}
		}
	}

	# Array population
	# Pupulate arrays
	push(@growth,   $contrib->{'TxtNone'});
	push(@complete, $contrib->{'TxtNone'});

	# List of layers
	$database->Pull(\$statement, "SELECT
	       	  LayerShort AS \"LayerShort\",
		  LayerLevel AS \"LayerLevel\"
		FROM Layer
		WHERE GalleryID=?
		ORDER BY LevelZero,
		         LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive",
	
	  $param->{'GID'}
	);

	# Loop through
	while ($res = $statement->fetchrow_hashref())
	{
		# Level
		$level = '-' x ($res->{'LayerLevel'} + 1);

		# Populate arrays
		# Growth needs room
		push(@growth,   $level . ' ' . $res->{'LayerShort'})
	         if ($res->{'LayerLevel'} < 6);

		# Complete is access to everything
		push(@complete, $level . ' ' . $res->{'LayerShort'});
	}

	# Finish
	$statement->finish();


	# Generate widgets
	# Scrolling lists and pop-up menu
	my $layerrem  = $cgi->popup_menu('LAYERREM', \@complete, $complete[0]);
	my $layerroot = $cgi->scrolling_list('LAYERROOT', \@growth, $growth[0], 10);

	# Textfield
	my $layeradd  = $cgi->textfield('LAYERADD', '', 16, 20);


	# Information display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplGalManage',
		LWLIST   => $links->{'LWLIST'},
		LGNEWS   => $links->{'LGNEWS'},
		LGLINK   => $links->{'LGLINK'},
		LGACCS   => $links->{'LGACCS'},
		LGLAYR   => $links->{'LGLAYR'},
		LGVIEW   => $links->{'LGVIEW'},
		WADDLST  => $layerroot,
		WREMLST  => $layerrem,
		WADDWRD  => $layeradd,
		WCREATE  => $cgi->submit($contrib->{'TxtCreate'}),
		WDELETE  => $cgi->submit($contrib->{'TxtDelete'})
	 );
	
	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Layer Generation
#
# Generates a nice, easy to use interface for moving back and forth
# between layers. 

sub LayerGenerate
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Gallery hash
	my $links    = shift;					# Hash of links

	my $link;						# Link to employ
	my $layers;						# Generation of layer
	my $res;						# Results list
	my $statement;						# Statement handles
	

	#####################
	# Program area	
	
	# Initialize
	$layers = '';

	# Determine link
	$link = ($param->{'TASK'} eq 'worklist')
	  ? $links->{'LWLIST'}
	  : $links->{'LGRSET'};


	# Layer display management
	# Display appropriate layers
	$database->Pull(\$statement, "SELECT
		  LayerID    AS \"LayerID\",
		  LayerShort AS \"LayerShort\",
		  LayerLevel AS \"LayerLevel\"
		FROM Layer
		WHERE GalleryID=?
		ORDER BY LevelZero, 
		         LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive,
			 LayerShort",

	  $param->{'GID'}
  	);

	# Confirm there is something to work with
	# Data check
	if ($res = $statement->fetchrow_hashref())
	{
		# Loop and manage
		do
		{
			# Append to list
			$layers .= $tmpl->Pass('TagLevel' . $res->{'LayerLevel'},
				MLINK => $link . '&GLAYER=' . $res->{'LayerID'},
				MNAME => $res->{'LayerShort'}
			 );

		# Loop until items are exhausted
		} while ($res = $statement->fetchrow_hashref());
	}

	# Checks
	$layers = (defined($layers))
	  ? $layers 
	  : $contrib->{'TagLayerMissing'};

	# Generate final template
	$layers = $tmpl->Pass('TmplGalLayers',
		LSTLAYERS => $layers,
		LRESET    => $link
	 );
}


#####################
# Quick Links Edit
#
# Link management allows for the additional sidebar entry that is used
# to show off certain links of interest to the author of a specific
# gallery.

sub LinkEntry
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $links    = shift;					# Hash of links

	my $res;						# Results
	my $statement;						# SQL statement

	my $link;						# Composit of links

	#####################
	# Program area	

	# Quick check
	return 0 unless (PermissionEditor($database, $param, $tmpl, $links));

	# Data management
	# Addition
	if (defined($param->{$contrib->{'TxtCreate'}}))
	{
		# Basic safety
		if ((defined($param->{'LNKNAME'}))
		 && (defined($param->{'LNKADD'}))
		 && (length($param->{'LNKNAME'}) > 3)
		 && ($param->{'LNKADD'} =~ /^http:\/\/.+/))
	 	{
			# Insert new link
			$database->Quick("INSERT INTO Link
				(PuppeteerLogin,
				 GalleryID,
				 LinkName,
				 LinkAddress)
				VALUES(?,?,?,?)",

			 $param->{'USER'},
			 $param->{'GID'},
			 $param->{'LNKNAME'},
			 $param->{'LNKADD'}
			);
		}
	}

	# List of current links	
	# Prepare and execute
	$database->Pull(\$statement, "SELECT
		 LinkID      AS \"LinkID\", 
		 LinkName    AS \"LinkName\", 
		 LinkAddress AS \"LinkAddress\"
		FROM Link
		WHERE PuppeteerLogin=?
		  AND GalleryID=?
		ORDER BY LinkName",

	 $param->{'USER'},
	 $param->{'GID'}
 	);


	# Initialize
	$link = '';
	
	# Loop and display current links
	while ($res = $statement->fetchrow_hashref())
	{
		# Display singular row
		unless (defined($param->{"LNK_$res->{LinkID}"}))
		{
			# Append to existing list
			$link .= $tmpl->Pass('TagGalLstLinks',
				MNAME   => $res->{'LinkName'},
				MLINK   => $res->{'LinkAddress'},
				WREVOKE => $cgi->checkbox(-name=>"LNK_$res->{LinkID}", -label=>'')
			 ) . "\n";
		}

		# User requested removal
		else
		{
			# Remove entry
			$database->Quick("DELETE FROM Link
				WHERE PuppeteerLogin=?
				  AND LinkID=?",

			 $param->{'USER'},
			 $res->{'LinkID'}
			);
		}
	}

	# End query
	$statement->finish();


	# Information display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplGalLinks',
		LWLIST   => $links->{'LWLIST'},
		LGNEWS   => $links->{'LGNEWS'},
		LGLINK   => $links->{'LGLINK'},
		LGACCS   => $links->{'LGACCS'},
		LGLAYR   => $links->{'LGLAYR'},
		LGVIEW   => $links->{'LGVIEW'},
		LSTLINKS => $link,
		WDELETE  => $cgi->submit($contrib->{'TxtDelete'}),
		WNAME    => $cgi->textfield('LNKNAME', '', 20, 20),
		WADDRESS => $cgi->textfield('LNKADD', 'http://', 40, 240),
		WCREATE  => $cgi->submit($contrib->{'TxtCreate'})
	 );
	
	# End form
	print $cgi->end_form(), "\n";
}


#####################
# News Entry
#
# Allows for the addition of news entries into the galleries front page
# as this is only a variant of the comments section, formatting will be
# more limited.

sub NewsEntry
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Gallery hash
	my $links    = shift;					# Hash of links

	my $wnews;						# News wiget
	my $wsubmit;						# Submit wiget

	#####################
	# Program area	

	# Quick check
	return 0 unless (PermissionEditor($database, $param, $tmpl, $links));


	# Cleanup
	# Cannot return directly
	$param->{'TASK'} = 'galview';

	
	# Generate wigits
	# News entry
	$wnews = $cgi->textarea(
		-name    => 'SAVE_NEWS',
		-rows    => 15,
		-columns => 75,
		-wrap    => 'SOFT',
		-default => ''
	 );

	# Submit button
	$wsubmit = $cgi->submit($contrib->{'TxtCreate'});

	
	# Information display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), EDIT_GALLERY=>'True'), "\n";


	# Display template
	$tmpl->Show('TmplGalNews',
		LWLIST   => $links->{'LWLIST'},
		LGNEWS   => $links->{'LGNEWS'},
		LGLINK   => $links->{'LGLINK'},
		LGACCS   => $links->{'LGACCS'},
		LGLAYR   => $links->{'LGLAYR'},
		LGVIEW   => $links->{'LGVIEW'},
		WNEWS    => $wnews,
		WSUBMIT  => $wsubmit
	 );
	
	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Permission
#
# A nice freindly way of reminding people that they are not where they
# should be at the moment.

sub Permission
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $links    = shift;					# Hash of links
	my $type     = shift;					# Type of restriction

	my $owner;						# Owner of gallery
	my $editor;						# Ediorial access
	my $janitor;						# The quicker picker upper
	my $puppet;						# Puppet handle


	#####################
	# Program area	

	# Pull permissions
	($owner,
	 $editor,
	 $janitor,
	 $puppet) = $database->DataGetGalleryAccess($param->{'GID'}, $param->{'USER'});


	# Quickly pass if true
	# Top level check
	if (($type eq 'owner')
	 && ($owner eq 'yes'))
	{
		# Return true
		return 1;
	}

	# Minor check
	elsif (($type eq 'editor')
	 && (($owner eq 'yes')
	  || ($owner eq 'yes')))
	{
		# Return true
		return 1;
	}


	# Failure is guarenteed at this point
	$tmpl->Show('TmplGalPermission',
		MPUPPET  => $puppet,
		LWLIST   => $links->{'LWLIST'},
		LGVIEW   => $links->{'LGVIEW'}
	 );

	# True if made it this far
	return 0;
}

# Shortcut for owners
sub PermissionOwner
{ return Permission(shift, shift, shift, shift, 'owner'); }

# Shortcut for owners
sub PermissionEditor
{ return Permission(shift, shift, shift, shift, 'editor'); }


#####################
# Work Detail
#
# Work detail, will handle the finer points of a works themselves.  This
# includes setting names, genres et cetera.

sub WorkDetail
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $links    = shift;					# Hash of links

	my $filter;						# Filter handler
	my $table;						# Table creation
	my $send;						# Mail handle
	my $wrd;						# Word and language handler

	my $res;						# Query results
	my $statement;						# Database query statement

	my $gid;						# Gallery id
	my $wid;						# Work id

	my $notify;						# Notification option

	my $exist;						# Existence
	my $gtype;						# Gallery type
	my $rating;						# Item rating
	my $thumb;						# Thumbnail
	my $votes;						# Votes

	my $select;						# Selection wiget
	my $unique;						# Unique attributes

	my @revs;						# Stored revisions
	

	#####################
	# Program area

	# Gallery type
	($gtype) = $database->DataGetGalleryType($param->{'GID'});


	# Primary set
	($wid) = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
	($gid) = $param->{'GID'}; 

	# Always set task
	$param->{'TASK'} = 'workview';
	

	# Update information
	if (defined($param->{'EDIT_WORK'}))
	{
		# Evaluate statement
		eval
		{
			# Common
			# Initiate handle
			$filter = new Ethereal::Filter();
			$wrd    = new Ethereal::Words();

			# Existence
			($exist) = $database->DataGetWorkID($param->{'WNAME'}, $param->{'GID'});


			# Main update
			# Description
			$param->{'WDESCR'} = $filter->StripHTML($param->{'WDESCR'});
			$param->{'WDESCR'} = substr(DBI::neat($param->{'WDESCR'}, 252), 1, -1);
			$param->{'WDESCR'} = (length($param->{'WDESCR'}) > 15) ? $param->{'WDESCR'} : undef;

			# Determin value
			$param->{'WNAME'}  = (!defined($exist)) ? $param->{'WNAME'} : $param->{'WORK'};

			# Image
			if ($gtype eq 'image')
			{
				# Generate thumbnail
				$thumb = "$database->{SYS}{SetGalleryLink}/$gid/$wid/thumb-work-$param->{WTHUMB}.$database->{SYS}{SetImgType}";

				# Update work
				$database->Write("UPDATE Work
					SET WorkName=?,
					    WorkDescription=?,
					    WorkImageThumbnail=?,
					    WorkTimestamp=LOCALTIMESTAMP
					WHERE WorkID=?
					  AND GalleryID=?",

				 $param->{'WNAME'},
				 $param->{'WDESCR'},
				 $thumb,
				 $wid,
				 $gid
				);
			}

			# Text
			else
			{
				# Archival
				if (defined($param->{'WARCHIVE'}))
				{
					# Pull old values prior to update
					my ($rlogin, $rname, $rtext, $rdate) = $database->DataGet("SELECT 
						  PuppeteerLogin     AS \"PuppeteerLogin\",
						  WorkName           AS \"WorkName\",
						  WorkText           AS \"WorkText\",
						  WorkTimestamp      AS \"WorkTimestamp\"
						FROM Work
						WHERE WorkID=?
						  AND GalleryID=?
						  AND WorkID > 1",

					 $wid,
					 $gid
					);

					# Create new entry
					$database->Write("INSERT INTO Revision
						(GalleryID,
						 WorkID,
						 RevisionTimestamp,
						 PuppeteerLogin,
						 RevisionName,
						 RevisionText)
						VALUES (?,?,?,?,?,?)",

					 $gid,
					 $wid,
					 $rdate,
					 $rlogin,
					 $rname,
					 $rtext
					);
				}
				
				# Remove revisions
				if ((defined($param->{'WREVISION'}))
				 && ($param->{'WREVISION'} ne $contrib->{'TxtAbstain'}))
			 	{
					# Remove revision
					$database->Write("DELETE FROM Revision
						WHERE GalleryID=?
						  AND WorkID=?
						  AND RevisionTimestamp=?",

					 $gid,
					 $wid,
					 $param->{'WREVISION'}
					);
				}

				
				# Preserve text
				my $preserve = (defined($param->{'WPRESERVE'})) ? 'yes' : 'no';

				# Safe HTML
				if ($preserve eq 'no')
				{
					# Initial filter
					$param->{'WTEXT'} = $filter->SafeHTML($param->{'WTEXT'});

					# Create hyperlinks
       		         		$param->{'WTEXT'} =~ s/(http:\/\/.*?)(\s|$)/<A HREF="$1" TARGET="_blank">$contrib->{TagHyperlink}<\/A> /gm;
       		         		$param->{'WTEXT'} =~ s/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/<A HREF="mailto:$1">$contrib->{TagEmail}<\/A> /gm;
				}

				# Additional HTML
				my $update = ($gtype ne 'journal') ? ', WorkTimestamp=LOCALTIMESTAMP': '';
				
				# Update work
				$database->Write("UPDATE Work
					SET WorkName=?,
					    WorkDescription=?,
					    WorkText=?,
					    WorkTextPreserve=?$update
					WHERE WorkID=?
					  AND GalleryID=?",

				 $param->{'WNAME'},
				 $param->{'WDESCR'},
				 $param->{'WTEXT'},
				 $preserve,
				 $wid,
				 $gid
				);


				# Image print formatting
				if ($gtype eq 'text')
				{
					# Create instance
					# Default setup
					my $page = new Ethereal::Home($database);

					# Directory name
					my $dir  = $database->{'SYS'}{'SetGalleryRoot'}
				                 . $page->{'SEP'}
				                 . $gid
						 . $page->{'SEP'}
						 . $wid;

					# Set basedire
					$page->{'BASEDIR'} = $dir;


					# Create a folder
					$page->DirCreate($dir) unless (-d $dir);
					

					# Export generation
					# Fetch external details
					my ($gname, $gauthor) = $database->DataGetGalleryBasics($gid);

					# Fetch and export work
					my ($export) = $database->DocumentGetGalleryExport();


					# Simplification
					$param->{'WTEXT'} =~ s/\n/<BR>/gs;
					
					# Search and replace
					$export =~ s/MAUTHOR/$gauthor/gs;
					$export =~ s/MGNAME/$gname/gs;
					$export =~ s/MGID/$gid/gs;
					$export =~ s/MWID/$wid/gs;
					$export =~ s/MBODY/$param->{'WTEXT'}/gs;
					$export =~ s/MGWORK/$param->{'WNAME'}/gs;
					$export =~ s/MTITLE/$param->{'WNAME'}/gs;

					
					# Write temporary file
					$page->FileWrite('export.html', $export);

					
					# Export to PDF
					# Evaluate clause
					eval
					{
						# Execute command
						system("$database->{SYS}{SetExport} $$database->{SYS}{SetExportOpt} $dir$page->{SEP}work.pdf $dir$page->{SEP}export.html");

					};

					# If block failed
					if ($@)
					{
						# Issue warning
						warn("Error in export: $@");
					}

					# Remove file
					$page->FileUnlink('export.html');
				}
			}

			# General update
			$database->Write("UPDATE Gallery
				SET GalleryUpdated=LOCALTIMESTAMP
				WHERE GalleryID=?", $gid);

			# Create reverse index
			$wrd->Reverse($database,
				$gid,
				$wid,
				$param->{'WNAME'},
				$param->{'WDESCR'},
				$param->{'WTEXT'}
			 );

			# Mail notification
			if (defined($param->{'NOTIFY'}))
			{
				# Verify for notification
				$database->Pull(\$statement, "SELECT
					p.PuppeteerName  AS \"PuppeteerName\",
					p.PuppeteerEmail AS \"PuppeteerEmail\"
					FROM Puppeteer p, GalleryNotify g
					WHERE g.PuppeteerLogin=p.PuppeteerLogin
					  AND g.GalleryID=?", $gid);


				# Notification applies
				if ($res = $statement->fetchrow_hashref())
				{
					# Fetch additional details
					# Basic gallery details
					my ($gname, $gauthor) = $database->DataGetGalleryBasics($gid);

					# Actual template
					my $message = $database->DocumentGetMailGallery();


					# Create instance
					$send = tie(*MAIL, 'Ethereal::Mail', $database);

					# Initial settings
					$send->SetSubject("Update to $param->{WNAME} for $gname by $gauthor");

					# Have at least one to
					$send->AddTo("$$database->{SYS}{SetSenderName} <$$database->{SYS}{SetSenderAddress}>");
					

					# Loop and correct
					do
					{
						# Recipients
						$send->AddBCC("$res->{PuppeteerName} <$res->{PuppeteerEmail}>");

					} while ($res = $statement->fetchrow_hashref());


					# Search and replace
					$send->SetSearch(
						MDESCR => $param->{'WDESCR'},
						MANAME => $gauthor,
						MGNAME => $gname,
						MWNAME => $param->{'WNAME'},
						LWORKA => "$database->{SYS}{LnkExtGallery}/view/$gid/$wid",
						LGALLA => "$database->{SYS}{LnkExtGallery}/view/$gid"
					);

					# Send out message
					print MAIL $message;

					# Close and send message
					close(MAIL);
				}

				# End search
				$statement->finish();
			}

			# Make sure things are done right
			$param->{'WORK'} = $param->{'WNAME'};
			
			# Commit changes
			$database->Commit();
		};


		# If block failed
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback();
		}
	}

	# Cleanup
	$param->Cleanup('WARCHIVE', 'WPRESERVE', 'NOTIFY');


	# Information gathering
	# Prepare and execute
	$database->Pull(\$statement, "SELECT 
		  WorkName           AS \"WorkName\",
		  WorkDescription    AS \"WorkDescription\",
		  WorkImage          AS \"WorkImage\",
		  WorkImageThumbnail AS \"WorkImageThumbnail\",
		  WorkText           AS \"WorkText\",
		  WorkTextPreserve   AS \"WorkTextPreserve\",
		  getDate(WorkTimestamp) AS \"WorkTimestamp\"
		FROM Work
		WHERE WorkID=?
		  AND GalleryID=?
		  AND WorkID > 1",

	 $wid,
	 $gid
	);


	# Results successful
	if ($res = $statement->fetchrow_hashref())
	{
		# Calculate rating
		($rating, $votes) = $database->DataGetWorkAvg($gid, $wid);

		# Initialize
		$unique = '';

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_WORK=>'True');


		# Image specific
		if ($gtype eq 'image')
		{
			# Display thumbnails
			my @thumb = qw(ful neg mid pos);

			# One at a time
			foreach $thumb (@thumb)
			{
				# Selected or not
				$select = ($res->{'WorkImageThumbnail'} =~ /$thumb/)
				  ? "<INPUT TYPE=\"radio\" NAME=\"WTHUMB\" VALUE=\"$thumb\" CHECKED>"
				  : "<INPUT TYPE=\"radio\" NAME=\"WTHUMB\" VALUE=\"$thumb\">";

				# Append
				$unique .= $tmpl->Pass('TagGalLstImage',
				   MLINK   => "$database->{SYS}{SetGalleryLink}/$gid/$wid/thumb-work-$thumb.$database->{SYS}{SetImgType}",
				   MHEIGHT => $database->{'SYS'}{'SetThumbnail'},
				   MWIDTH  => $database->{'SYS'}{'SetThumbnail'},
				   WSELECT => $select
				 ) . "\n";
			}

			# Mail template
			$unique = $tmpl->Pass('TmplGalLstImage',
				LSTIMAGE => $unique
			 );
		}

		# Text specific
		else
		{
			# Generate widget
			# Archiving
			my $optarchive  = '';

			# Preservation of text
			my $preserve = ($res->{'WorkTextPreserve'} eq 'yes')
			     ? $cgi->checkbox(-name=>'WPRESERVE', -label=>'', -checked=>'on')
			     : $cgi->checkbox(-name=>'WPRESERVE', -label=>'');

			# Text body
			my $wtext = $cgi->textarea(
					-name    => 'WTEXT',
					-rows    => 15,
					-columns => 75,
					-wrap    => 'SOFT',
					-default => $res->{'WorkText'}
			 );

			
			# Text-only specific options
			# Journals do not require this
			if ($gtype eq 'text')
			{

				# Revisions
				# Pull a list of revisions
				$database->GetList(\@revs, "SELECT RevisionTimestamp
					FROM Revision
					WHERE GalleryID=?
					  AND WorkID=?
					ORDER BY RevisionTimestamp",
	
				 $gid,
				 $wid
				);

				# Implemention of safety
				unshift(@revs, $contrib->{'TxtAbstain'}) if (defined($revs[0]));


				# Generate wigets
				# Archival
				my $warchive = $cgi->checkbox(-name=>'WARCHIVE', -label=>'');

				# Deprecation
				my $wdeprecate = (defined($revs[0]))
				     ? $cgi->popup_menu('WREVISION', \@revs, $revs[0])
				     : $contrib->{'TagGalDeprecate'};


				# Generate option
				$optarchive = $tmpl->Pass('TmplGalLstTextArchive',
					WARCHIVE   => $warchive,
					WDEPRECATE => $wdeprecate
				  );
			}	
			
			# Generate unique template
			$unique .= $tmpl->Pass('TmplGalLstText',
				WWTEXT    => $wtext,
				WPRESERVE => $preserve,
				WOARCHIVE  => $optarchive
			 );
			
		}

		# Generate Widget
		# Notification
		my $notify = $cgi->checkbox(-name=>'NOTIFY', -label=>'');
		
		# Title
		my $name = $cgi->textfield('WNAME', $res->{'WorkName'}, 25, 30);
		
		# Description
		my $descr  = $cgi->textarea(
			-name    => 'WDESCR',
			-rows    => 3,
			-columns => 75,
			-wrap    => 'SOFT',
			-default => $res->{'WorkDescription'}
		 );

		# Standard buttons
		my $submit = $cgi->submit($contrib->{'TxtUpdate'});
		my $reset  = $cgi->reset($contrib->{'TxtReset'});


		# Display template
		$tmpl->Show('TmplGalWork',
			LWLIST   => $links->{'LWLIST'},
			LGNEWS   => $links->{'LGNEWS'},
			LGLINK   => $links->{'LGLINK'},
			LGACCS   => $links->{'LGACCS'},
			LGLAYR   => $links->{'LGLAYR'},
			LGVIEW   => $links->{'LGVIEW'},
			MWNAME   => $res->{'WorkName'},
			MOLVOTES => $votes,
			MRATINGS => $rating,
			MUPDATED => $res->{'WorkTimestamp'},
			MUNIQUE  => $unique,
			WWNAME   => $name,   
			WWNOTIFY => $notify,
			WWDESCR  => $descr,
			WUPDATE  => $submit,
			WRESET   => $reset
			
		 );
		
		# End form
		print $cgi->end_form, "\n";
	}

	# End query
	$statement->finish();
}


#####################
# Work Summary
#
# Will list all of the works, while allowing the user to add and remove
# them as need be.  This will be similiar to the interface for galleries
# only work specific.

sub WorkSummary
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $links    = shift;					# Hash of links
	my $upload   = shift;					# Upload handle
	my $remove   = shift;					# Items to remove

	my $image;						# Image library
	my $leech;						# Leeching module
	my $option;						# Option handler
	my $page;						# File handler

	my $message;						# Notice message

	my $gid;						# Gallery ID
	my $lid;						# Layer ID
	my $wid;						# Work ID

	my $fsort;						# Final sort order
	my $gsort;						# Gallery sorting
	my $gtype;						# Gallery type
	my $work;						# Individual works
	
	my $editor;						# Editor privs
	my $janitor;						# Janitoral privs
	my $owner;						# Owner privs

	my $contents;						# File contents
	my $layers;						# Layer management
	my $err;						# Write error message
	my $mime;						# Mime type
	my $name;						# File extension

	my @works;						# Array of works
	my @pass;						# Passage of information


	#####################
	# Program area	

	# Gallery type
	($gtype, $_, $gsort) = $database->DataGetGalleryType($param->{'GID'});

	# Pull permissions
	($owner, $editor, $janitor) = $database->DataGetGalleryAccess($param->{'GID'}, $param->{'USER'});


	# Information handling
	# Additions with names greater then 3 
	if ((defined($param->{$contrib->{'TxtCreate'}}))
	 && (CheckString($param->{'ADD_WORK'}, 2)))
	{
		# Duplicate check
		($wid)  = $database->DataGetWorkID($param->{'ADD_WORK'}, $param->{'GID'});

		# Safe to proceed
		unless (defined($wid))
		{
			# Evalutate
			eval
			{
				# Database work
				# Create database entry
				$database->Write("INSERT INTO Work
					(GalleryID,
					 WorkLayer,
					 WorkName,
					 WorkSort,
					 PuppeteerLogin)
					VALUES (?,?,?,getSort(?),?)", 

				 $param->{'GID'},
				 $param->{'GLAYER'},
				 $param->{'ADD_WORK'},
				 $param->{'GID'},
				 $param->{'USER'}
				);

				# Update gallery
				$database->Write("UPDATE Gallery
					SET GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?",

				 $param->{'GID'}
				);


				# Retreive Work and Gallery ID
				($wid) = $database->DataGet("SELECT CURRVAL('seqWork')");
				($gid) = $param->{'GID'};


				# File work
				# Image galleries only
				if ($gtype eq 'image')
				{
					# File exists
					if (defined($upload))
					{
						# Basics
						$mime = $cgi->uploadInfo($param->{'UPLOAD_WORK'})->{'Content-Type'};
						$name = $param->{'UPLOAD_WORK'};
					
						# Check mime types
						# Acceptable mime type
						if ($mime =~ /^image/i)
						{
							# Create instances
							# With safe defaults
							$page  = new Ethereal::Home($database,  "$database->{SYS}{SetGalleryRoot}/0");
							$image = new Ethereal::Image($database, "$database->{SYS}{SetGalleryRoot}/0");

							# Leeching and writing
							$leech = new Ethereal::Leech($database, $image, $page, $param, $contrib);

							# Setting of wid
							$leech->{'WID'} = $wid;
							$leech->{'MSG'} = 1;

							# Storing of file
							$message = $leech->Store($name, $upload);
						}

						# Improper mime type
						else
						{
							# Warning 
							$message = $contrib->{'MsgMime'};
						}
					}
				}

				# Nothing to create
				else
				{
					# Message of success
					$message = $contrib->{'MsgCreated'};
				}

				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}
		else
		{
			# Name exists
			$message = $contrib->{'MsgExists'};
		}
	}

	# Removal
	if ((defined($param->{$contrib->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Loop through and remove
		foreach $wid (@{$remove})
		{
		
			# Determine Work/Gallery ID
			($wid)  = $database->DataGetWorkID($wid, $param->{'GID'});
			($gid)  = $param->{'GID'};
			($lid)  = $param->{'GLAYER'};

			# Remove
			if (defined($wid))
			{
				# Initialize counter
				my $count = 1;

				# Pull list of works
				$database->GetListWorkSort(\@works, $gid);


				# Evaluate block
				eval
				{
					# Database entry
					$database->Write("DELETE FROM Work
						WHERE WorkID=?
						  AND GalleryID=?", 
	
					 $wid, 
					 $gid
					);
	
					# Loop and re-assign
					foreach $work (@works)
					{
						# Update individual entries to new order
						$database->Write("UPDATE Work
							SET WorkSort=?
							WHERE WorkID=?
							  AND WorkLayer=?
							  AND GalleryID=?",
							  
						
						 $count,
						 $work,
						 $lid,
						 $gid
					 	);

						# Increment
						$count++;
					}
				

					# Update gallery
					$database->Write("UPDATE Gallery
						SET GalleryUpdated=LOCALTIMESTAMP
						WHERE GalleryID=?",
	
					 $gid
					);


					# Directory structure
					if ($gtype eq 'image')
					{
						# Create instances
						$page  = new Ethereal::Home($database, "$database->{SYS}{SetGalleryRoot}/$gid/$wid");
	
						# Remove directory
						$page->DirClear($page->{'BASEDIR'})
						  if (-d $page->{'BASEDIR'});
					}
	
					# Define message
					$message = $contrib->{'MsgRemoved'};
	
					# Commit changes
					$database->Commit();
				};
	
				# If block failed
				if ($@)
				{
					# Issue warning
					warn("Transaction aborted: $@");
	
					# Undo incomplete changes
					$database->Rollback();
				}
			}
		}
	}

	# Movement
	if ((($owner eq 'yes')
	 || ($editor eq 'yes'))
	 && ((defined($param->{'MUPS'}))
	 || (defined($param->{'MDWN'}))))
	{
		# Determine Work/Gallery ID
		($wid)  = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
		($gid)  = $param->{'GID'};
		($lid)  = $param->{'GLAYER'};

		# Remove
		if (defined($wid))
		{
			# Initialize counter
			my $count  = 1;

			# Current and level
			my $icnt;
			my $lvl;
			my $lchk;

			# Obstacle
			my $obs;


			# Operation used in query
			my $op     = (defined($param->{'MUPS'})) ? '+' : '-';

			# Affect of movement
			$lvl       = (defined($param->{'MUPS'})) ? $param->{'MUPS'} : $param->{'MDWN'};
			$lvl       = ((defined($lvl)) && ($lvl < 15)) ? $lvl : 1;

			# Affects the layer
			$lchk      = (defined($lid)) ? '=' . $lid : 'IS NULL';


			# Current sort number
			my ($wsrt) = $database->DataGetWorkSort($wid, $gid);
			

			# Continue until exhausted
			for ($icnt=$lvl; $icnt > 0; $icnt--)
			{
				# Premature exist
				last if (defined($obs));

				# What is in the way
				($obs)  = $database->DataGet("SELECT WorkID
					FROM Work
					WHERE WorkSort=(? $op $icnt)::INT
					  AND WorkLayer $lchk
					  AND GalleryID=?
					  AND WorkID > 1",
				 $wsrt,
				 $gid
				);
			}

			# Safety	
			$obs = (defined($obs)) ? $obs : 0;


			# Pull list of works
			$database->GetList(\@works, "SELECT WorkID
	 			 FROM Work
				WHERE WorkLayer $lchk
				  AND GalleryID=?
				  AND WorkID > 1
				ORDER BY WorkSort", $gid);


			# Evaluate block
			eval
			{
				# Loop and re-assign
				foreach $work (@works)
				{
					# Loop changes
					# No change for ends
					last if ($obs == 0);

					# Skip on target
					next if ($work == $wid);


					# Insert before current entry
					if (($work == $obs) && (defined($param->{'MDWN'})))
					{
						# Push in entry ahead
						$database->Write("UPDATE Work
							SET WorkSort=?
							WHERE WorkID=?
							  AND GalleryID=?",
					
						 $count,
						 $wid,
						 $gid
				 		);

						# Increment
						$count++;
					}

					
					# Update individual entries to new order
					$database->Write("UPDATE Work
						SET WorkSort=?
						WHERE WorkID=?
						  AND GalleryID=?",
					
					 $count,
					 $work,
					 $gid
				 	);

					# Increment
					$count++;

					# Insert after current entry
					if (($work == $obs) && (defined($param->{'MUPS'})))
					{
						# Push in entry ahead
						$database->Write("UPDATE Work
							SET WorkSort=?
							WHERE WorkID=?
							  AND GalleryID=?",
					
						 $count,
						 $wid,
						 $gid
				 		);

						# Increment
						$count++;
					}

				}
				
				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}
	}

	# Always cleanup
	$param->Cleanup('CONFIRM', 'MDWN', 'MUPS');

	# Create instance
	$option = new Ethereal::Option();


	# Determine sort order
	$fsort = $option->GallerySortReal($gsort);


	# Quick preperation
	# Passage of information
	push(@pass, $param->{'GID'});

	# Layers
	$layers = (defined($param->{'GLAYER'})) ? 'AND WorkLayer=? ' : 'AND WorkLayer IS NULL '; 

	# Additional parameter
	push(@pass, $param->{'GLAYER'})
          if (defined($param->{'GLAYER'}));


	# Retrieve list
	if (($owner eq 'yes')
	 || ($editor eq 'yes'))
 	{
		# Owners and editors
		$database->GetList(\@works, "SELECT WorkName
			 FROM Work 
			 WHERE GalleryID=? $layers
			   AND WorkID > 1 
			 ORDER BY $fsort", 

		 @pass,
		);
	}
	else
	{
		# Normal contributors
		$database->GetList(\@works, "SELECT WorkName
			 FROM Work 
			 WHERE GalleryID=? $layers
			   AND WorkID > 1 
			   AND PuppeteerLogin=?
			 ORDER BY $fsort", 

		 @pass,
		 $param->{'USER'}
		);
	}


	# Information gathering
	# Generate final template
	$layers = LayerGenerate($database, $param, $tmpl, $contrib, $links);

	# Tell people where they are
	my ($ident) = (defined($param->{'GLAYER'}))
	  ? $database->DataGet("SELECT LayerShort
		  FROM Layer
		  WHERE GalleryID=?
		    AND LayerID=?", $param->{'GID'}, $param->{'GLAYER'})
	  : $database->DataGet("SELECT GalleryName
		  FROM Gallery
		  WHERE GalleryID=?", $param->{'GID'});


	# Widget generation
	# Work list within layer
	my $listing = $cgi->scrolling_list('WORK', \@works, $param->{'WORK'}, 10, 'true');

	# Details, create  and delete buttons	
	my $create  = $cgi->submit($contrib->{'TxtCreate'});
	my $delete  = $cgi->submit($contrib->{'TxtDelete'});
	my $details = $cgi->submit($contrib->{'TxtDetails'}); 

	# Confirmation checkbox
	my $confirm = $cgi->checkbox(-name=>'CONFIRM', -label=>'');

	# Creation wigets
	my $wname   = $cgi->textfield('ADD_WORK', '', 25, 30);


	# Arrow generations
	# Initialize
	my $arrows = '';

	# Arror generation
	if (($gsort eq 'manual')
	 && (($owner eq 'yes')
	  || ($editor eq 'yes')))
	{
		# Generate arrows list
		$arrows  .= 
		  $tmpl->Pass('TagSortBtn', MNAME=>'MDWN', MVAL=>'1',  MLINK=>$contrib->{'TagArrowDown'}) .
		  $tmpl->Pass('TagSortBtn', MNAME=>'MDWN', MVAL=>'10', MLINK=>$contrib->{'TagArrowDblDown'}) .
		  $tmpl->Pass('TagSortBtn', MNAME=>'MUPS', MVAL=>'1',  MLINK=>$contrib->{'TagArrowDblUp'}) .
		  $tmpl->Pass('TagSortBtn', MNAME=>'MUPS', MVAL=>'10', MLINK=>$contrib->{'TagArrowUp'});
	}


	# Additions
	# Initialize
	$upload = '';

	# Additions
	if  ($gtype eq 'image')
 	{
		# Generate upload template
		$upload = $tmpl->Pass('TmplGalWorkUpload' ,
			WLEECH  => $cgi->textfield('LEECH_WORK', 'http://', 40, 240),
		        WUPLOAD => $cgi->filefield('UPLOAD_WORK', '', 40, 140)
		 );
	}

	# Precaution
	$message = (defined($message)) ? $message : '';
	

	# Information display
	# Forms 
	print $cgi->start_multipart_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplGalWorkList',
		LWLIST   => $links->{'LWLIST'},
		LGNEWS   => $links->{'LGNEWS'},
		LGLINK   => $links->{'LGLINK'},
		LGACCS   => $links->{'LGACCS'},
		LGLAYR   => $links->{'LGLAYR'},
		LGVIEW   => $links->{'LGVIEW'},
		MARROWS  => $arrows,
		MLAYERS  => $layers,
		MNAME    => $ident,
		MMESSAGE => $message,
		MUPLOAD  => $upload,
		WLISTING => $listing,
		WCREATE  => $create,
		WDELETE  => $delete,
		WDETAILS => $details,
		WCONFIRM => $confirm,
		WNNAME   => $wname	
	  );
	

	# End form
	print $cgi->end_form(), "\n";
}
